/***************************************************
  This is an example for our Adafruit FONA Cellular Module

  Designed specifically to work with the Adafruit FONA
  ----> http://www.adafruit.com/products/1946
  ----> http://www.adafruit.com/products/1963
  ----> http://www.adafruit.com/products/2468
  ----> http://www.adafruit.com/products/2542

  These cellular modules use TTL Serial to communicate, 2 pins are
  required to interface
  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ****************************************************/

/*
THIS CODE IS STILL IN PROGRESS!

Open up the serial console on the Arduino at 115200 baud to interact with FONA

Note that if you need to set a GPRS APN, username, and password scroll down to
the commented section below at the end of the setup() function.
*/
#include "Adafruit_FONA.h"

#define FONA_RX 2
#define FONA_TX 3
#define FONA_RST 9

// this is a large buffer for replies
char replybuffer[255];
char callerIDbuffer[32];   //we'll store the SMS sender number in here
// We default to using software serial. If you want to use hardware serial
// (because softserial isnt supported) comment out the following three lines 
// and uncomment the HardwareSerial line
#include <SoftwareSerial.h>
SoftwareSerial fonaSS = SoftwareSerial(FONA_RX, FONA_TX);
SoftwareSerial *fonaSerial = &fonaSS;

// Hardware serial is also possible!
//  HardwareSerial *fonaSerial = &Serial1;

// Use this for FONA 800 and 808s
//Adafruit_FONA fona = Adafruit_FONA(FONA_RST);
// Use this one for FONA 3G
Adafruit_FONA_3G fona = Adafruit_FONA_3G(FONA_RST);

uint8_t readline(char *buff, uint8_t maxbuff, uint16_t timeout = 0);

uint8_t type;

void setup() 
{
  pinMode(A0, OUTPUT);
  digitalWrite(A0, LOW);
  
  while (!Serial);

  Serial.begin(115200);
  Serial.println(F("FONA basic test"));
  Serial.println(F("Initializing....(May take 3 seconds)"));

  fonaSerial->begin(4800);
  if (! fona.begin(*fonaSerial)) {
    Serial.println(F("Couldn't find FONA"));
    while (1);
  }
  type = fona.type();
  Serial.println(F("FONA is OK"));
  Serial.print(F("Found "));
  switch (type) {
    case FONA800L:
      Serial.println(F("FONA 800L")); break;
    case FONA800H:
      Serial.println(F("FONA 800H")); break;
    case FONA808_V1:
      Serial.println(F("FONA 808 (v1)")); break;
    case FONA808_V2:
      Serial.println(F("FONA 808 (v2)")); break;
    case FONA3G_A:
      Serial.println(F("FONA 3G (American)")); break;
    case FONA3G_E:
      Serial.println(F("FONA 3G (European)")); break;
    default: 
      Serial.println(F("???")); break;
  }
  
  // Print module IMEI number.
  char imei[16] = {0}; // MUST use a 16 character buffer for IMEI!
  uint8_t imeiLen = fona.getIMEI(imei);
  if (imeiLen > 0) {
    Serial.print("Module IMEI: "); Serial.println(imei);
  } 
}

void loop() 
{
  delay(2000);
  if(true==ReadSMS(0))
  {
    DeleteSMS(0);
  }
}


void DeleteSMS(unsigned char sn)
{
  if(fona.deleteSMS(sn))
  {
    Serial.println(F("No SMS to delete"));
  }
  else
  {
    Serial.println(F("Delete a SMS"));
  }
  Serial.println(F("Ready to recive the SMS"));
}


boolean ReadSMS(unsigned char sn)
{
  uint16_t smslen;
  boolean read_sms_sucess_flag=false;
  
  Serial.print(F("***** Read SMS #")); 
  Serial.print(sn);
  Serial.println(F(" *****"));
  
  if (fona.getSMSSender(0, callerIDbuffer, 250)) 
   {
      Serial.print("From: ");
      Serial.println(callerIDbuffer);
    }
  
  if (! fona.readSMS(sn, replybuffer, 250, &smslen)) 
  { // pass in buffer and max len!
    Serial.println("Failed!");
  }
  else
  {
    Serial.println(replybuffer);
    read_sms_sucess_flag=true;
    
    if(replybuffer[0]=='1')
    {
      digitalWrite(A0, HIGH);
      if (!fona.sendSMS(callerIDbuffer, "LED is ON!"))
      {
        Serial.println(F("Failed"));
      } 
      else 
        {
           Serial.println(F("Sent!"));
        } 
    }
    else if(replybuffer[0]=='0')
    {
      digitalWrite(A0, LOW);
      if (!fona.sendSMS(callerIDbuffer, "LED is OFF!"))
      {
        Serial.println(F("Failed"));
      }
      else 
       {
         Serial.println(F("Sent!"));
       }
    }
    
  }
  Serial.println(F("*****"));
  return read_sms_sucess_flag;
}

void flushSerial() {
  while (Serial.available())
    Serial.read();
}

char readBlocking() {
  while (!Serial.available());
  return Serial.read();
}
uint16_t readnumber() {
  uint16_t x = 0;
  char c;
  while (! isdigit(c = readBlocking())) {
    //Serial.print(c);
  }
  Serial.print(c);
  x = c - '0';
  while (isdigit(c = readBlocking())) {
    Serial.print(c);
    x *= 10;
    x += c - '0';
  }
  return x;
}

uint8_t readline(char *buff, uint8_t maxbuff, uint16_t timeout) {
  uint16_t buffidx = 0;
  boolean timeoutvalid = true;
  if (timeout == 0) timeoutvalid = false;

  while (true) {
    if (buffidx > maxbuff) {
      //Serial.println(F("SPACE"));
      break;
    }

    while (Serial.available()) {
      char c =  Serial.read();

      //Serial.print(c, HEX); Serial.print("#"); Serial.println(c);

      if (c == '\r') continue;
      if (c == 0xA) {
        if (buffidx == 0)   // the first 0x0A is ignored
          continue;

        timeout = 0;         // the second 0x0A is the end of the line
        timeoutvalid = true;
        break;
      }
      buff[buffidx] = c;
      buffidx++;
    }

    if (timeoutvalid && timeout == 0) {
      //Serial.println(F("TIMEOUT"));
      break;
    }
    delay(1);
  }
  buff[buffidx] = 0;  // null term
  return buffidx;
}
